# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import csv
import json
import os
import sys


def _load_data_from_csv(path):
    with open(path) as csvfile:
        csvdata = [row for row in csv.reader(csvfile)]
    inventory = {}
    # NOTE(pas-ha) convert to structure similar to JSON inventory
    for entry in csvdata:
        mac = entry[0]
        hostname = entry[10]
        ip = entry[11]
        inventory[hostname] = {
            'nics': [{'mac': mac}],
            'name': hostname,
            'ipv4_address': ip
        }
    return inventory


def _load_data_from_json(path):
    with open(path) as jsonfile:
        inventory = json.load(jsonfile)
    return inventory


def main(argv):
    # first item is the inventory_dhcp setting
    # second item is the inventory_dhcp_static_ip setting
    inventory_dhcp = (argv[0] == 'True' or argv[0] == 'true')
    inventory_dhcp_static_ip = (argv[1] == 'True' or argv[1] == 'true')

    if not inventory_dhcp:
        # nothing to validate
        sys.exit(0)

    # load data from json file
    if os.path.exists('/tmp/baremetal.json'):
        inventory = _load_data_from_json('/tmp/baremetal.json')
    # load data from csv file
    elif os.path.exists('/tmp/baremetal.csv'):
        try:
            inventory = _load_data_from_csv('/tmp/baremetal.csv')
        except Exception:
            # try load *.csv as json for backward compatibility
            inventory = _load_data_from_json('/tmp/baremetal.csv')
    else:
        print('ERROR: Inventory file has not been generated')
        sys.exit(1)

    # now check that we only have these entries in leases file
    leases = []
    if not os.path.exists('/var/lib/misc/dnsmasq.leases'):
        if not os.path.exists('/var/lib/dnsmasq/dnsmasq.leases'):
            print('ERROR: dnsmasq leases file has not been generated')
            sys.exit(1)
        else:
            dns_path = '/var/lib/dnsmasq/dnsmasq.leases'
    else:
        dns_path = '/var/lib/misc/dnsmasq.leases'

    with open(dns_path) as csvfile:
        leases_reader = csv.reader(csvfile, delimiter=' ')
        for row in leases_reader:
            leases.append(row)

    # first we test number of entries
    if len(leases) != len(inventory):
        print('ERROR: Number of entries do not match with inventory')
        sys.exit(1)

    # then we check that all macs and hostnames are present
    for value in inventory.values():
        # NOTE(pas-ha) supporting only single nic
        mac = value['nics'][0]['mac']
        hostname = value['name']
        ip = value['ipv4_address']

        # mac check
        for lease_entry in leases:
            if lease_entry[1] == mac:
                break
        else:
            print('ERROR: No mac found in leases')
            sys.exit(1)

        # hostname check
        for lease_entry in leases:
            if lease_entry[3] == hostname:
                # if we use static ip, we need to check that ip matches
                # with hostname in leases
                if inventory_dhcp_static_ip:
                    if lease_entry[2] != ip:
                        print('ERROR: IP does not match with inventory')
                        sys.exit(1)
                break
        else:
            print('ERROR: No hostname found in leases')
            sys.exit(1)

    sys.exit(0)


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python
# coding: utf-8 -*-

# (c) 2015, Hewlett-Packard Development Company, L.P.
#
# This module is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

DOCUMENTATION = '''
---
module: network_metadata
short_description: Returns a config-drive network-metadata dictionary
extends_documentation_fragment: openstack
'''


def main():
    argument_spec = dict(
        ipv4_address=dict(required=False),
        ipv4_gateway=dict(required=False),
        ipv4_interface_mac=dict(required=False),
        ipv4_nameserver=dict(required=False, type='list'),
        ipv4_subnet_mask=dict(required=False),
        vlan_id=dict(required=False),
        network_mtu=dict(required=False),
        nics=dict(required=False, type='list'),
        node_network_data=dict(required=False, type='dict')
    )

    module = AnsibleModule(argument_spec)  # noqa: F405

    network_metadata = module.params['node_network_data']
    if not network_metadata:
        links = []
        networks = []

        if module.params['ipv4_interface_mac']:
            nic_id = module.params['ipv4_interface_mac']

            if module.params['vlan_id']:
                nic_id = 'vlan-%s' % module.params['ipv4_interface_mac']

                links.append({
                    'id': nic_id,
                    'type': 'vlan',
                    'vlan_id': module.params['vlan_id'],
                    'vlan_link': module.params['ipv4_interface_mac'],
                    'vlan_mac_address': module.params['ipv4_interface_mac'],
                })

            links.append({
                'id': module.params['ipv4_interface_mac'],
                'type': 'phy',
                'ethernet_mac_address': module.params['ipv4_interface_mac'],
                'mtu': module.params['network_mtu']
            })

            for nic in module.params['nics']:
                if nic['mac'] == module.params['ipv4_interface_mac']:
                    networks.append({
                        'id': 'ipv4-%s' % nic_id,
                        'link': nic_id,
                        'type': 'ipv4',
                        'ip_address': module.params['ipv4_address'],
                        'netmask': module.params['ipv4_subnet_mask'],
                        'dns_nameservers': module.params['ipv4_nameserver'],
                        'routes': [{
                            'network': '0.0.0.0',
                            'netmask': '0.0.0.0',
                            'gateway': module.params['ipv4_gateway']
                        }]
                    })
        else:
            for i, nic in enumerate(module.params['nics']):
                nic_id = nic['mac']
                if module.params['vlan_id']:
                    nic_id = 'vlan-%s' % nic['mac']

                    links.append({
                        'id': nic_id,
                        'type': 'vlan',
                        'vlan_id': module.params['vlan_id'],
                        'vlan_link': nic['mac'],
                        'vlan_mac_address': nic['mac']
                    })

                links.append({
                    'id': nic['mac'],
                    'type': 'phy',
                    'ethernet_mac_address': nic['mac'],
                    'mtu': module.params['network_mtu']
                })

                if i == 0:
                    networks.append({
                        'id': 'ipv4-%s' % nic_id,
                        'link': nic_id,
                        'type': 'ipv4',
                        'ip_address': module.params['ipv4_address'],
                        'netmask': module.params['ipv4_subnet_mask'],
                        'dns_nameservers': module.params['ipv4_nameserver'],
                        'routes': [{
                            'network': '0.0.0.0',
                            'netmask': '0.0.0.0',
                            'gateway': module.params['ipv4_gateway']
                        }]
                    })
                else:
                    networks.append({
                        'id': 'ipv4-dhcp-%s' % nic_id,
                        'link': nic_id,
                        'type': 'ipv4_dhcp',
                    })

        services = []
        if module.params['ipv4_nameserver']:
            for item in module.params['ipv4_nameserver']:
                services.append({
                    'type': 'dns',
                    'address': item
                })

        network_metadata = {
            'links': links,
            'networks': networks,
            'services': services
        }
    facts = {'network_metadata': network_metadata}

    module.exit_json(changed=False, ansible_facts=facts)


# this is magic, see lib/ansible/module_common.py
from ansible.module_utils.basic import *  # noqa: E402

if __name__ == '__main__':
    main()
